import cv2
import sys, string

def get_input():
    in_ = []
    while True:
        k = cv2.waitKey()
        if k==27 or k==13:
            break
        elif k==8:
            try:
                in_.pop()
                sys.stdout.write("\b \b")
                sys.stdout.flush()
            except IndexError:
                continue
        else:
            char = chr(k)
            if char not in string.printable:
                continue
            print(chr(k), end='', flush=True)
            in_.append(chr(k))
    print("")

    return "".join(in_)

def ident_item(im, output="", save=True):
    cv2.imshow("Identify this:", im)

    identity = get_input()
    if save:
        cv2.imwrite(output+identity+".png", im)
    return identity

def ident_num(im):
    cv2.imshow("Identify this:", im)

    identity = get_input()

    return identity



import cv2
import logging
import os

from sheets.core import (
    find_numbers, find_icons, prepare_icon, load_numbers, load_icons, match_item, 
    ocr, guess_resolution
    )

DEBUG = False

class Stockpile():
    RESOLUTIONS = {
        "1920x1080": (42,32),
        "2560x1440": (56,43),
        "2560x1440x2": (56,42)
    }

    @classmethod
    def bool_size(cls, rectangle, resolution):
        try:
            size = cls.RESOLUTIONS[resolution]
        except KeyError:
            raise NotImplementedError(f"{resolution} not available in class Stockpile")

        _,_,w,h = cv2.boundingRect(rectangle)
        return (w,h) == size

    @classmethod
    def get_resolution(cls, image, rects):
        return guess_resolution(image, rects)

    def __init__(self, image, 
                 resolution=None, 
                 min_err=0.03, 
                 number_identities=None,
                 icon_identities=None,
                 numbers=None,
                 icons=None,
                 parse=True):
        self.image = image
        self.min_err = min_err
        self.data = dict()
        self.unidentified = []

        if numbers:
            self.numbers=numbers
        else:
            self.numbers=find_numbers(self.image)

        if resolution:
            self.resolution = resolution
        else:
            self.resolution = guess_resolution(image, self.numbers)
        assert self.resolution in Stockpile.RESOLUTIONS

        if number_identities:
            self.number_identities = number_identities
        else:
            self.number_identities = load_numbers(resolution=self.resolution)
        if icon_identities:
            self.icon_identities = icon_identities
        else:
            self.icon_identities = load_icons(resolution=self.resolution)

        self.icons=find_icons(self.numbers, resolution=self.resolution)
        self.names = list(self.icon_identities.keys())
        self.icon_arrays = list(self.icon_identities.values())

        if parse: self.parse()

        for k, v in self.data.items():
            self.data[k] = v.replace("k+", "000")

    def parse(self):
        for number, icon in zip(self.numbers, self.icons):
            x,y,w,h = cv2.boundingRect(number)
            number_image = self.image[y:y+h,x:x+w]

            x,y,w,h = cv2.boundingRect(icon)
            icon_image = self.image[y:y+h,x:x+w]

            icon_image = prepare_icon(icon_image)
            err, item_index = match_item(icon_image, self.icon_arrays)

            if err is None:
                logging.error(f"There are no icons to match with.")
                self.unidentified.append(self.image[y:y+h,x:x+w].copy())
                continue

            if err >= self.min_err:
                logging.info(f"Could not identify {(x,y,w,h)}, with err:{err}")
                self.unidentified.append(self.image[y:y+h,x:x+w].copy())
                continue

            name = os.path.basename(self.names[item_index])
            name, ext = os.path.splitext(name)
            val = ocr(number_image, self.number_identities, resolution=self.resolution)
            self.data[name] = val 



#!/usr/bin/env python
import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(name='foxhole-sheets',
      version='3.0.0',
      description='Foxhole Stockpile Parsing',
      long_description=README,
      long_description_content_type="text/markdown",
      url="https://github.com/miles-igd/foxhole-sheets",
      author="miles-igd",
      license="MIT",
      include_package_data=True,
      install_requires=[
      'opencv-python'
      ],
      packages=['sheets']
     )